/**
  General-Purpose Graphics Processing Unit (GPGPU)
  Demo Application
  
  Orion Sky Lawlor, olawlor@acm.org, 2008-04-21 (Public Domain)
*/
#include "ogl/gpgpu.h" /* Orion's GPGPU library */
#include "ogl/glsl.cpp" /* for easy linking, just #include the library code */

int main(int argc,char *argv[]) {
	gpgpu_init();
	
	/* Upload a small lookup-table texture */
	const static float my_colors[4*3*2]={
	/* floats are R,G,B,A colors for each pixel: 3 pixels wide, 2 pixels high */
		1,0,0,0,   0,1,0,0,  0,0,1,0,
		0,0,0,0,   1,1,1,1,  0.5,0.5,0.5,0
	};
	static gpgpu_texture2D *table=new gpgpu_texture2D(3,2,my_colors);
	
	/* Make an output texture */
	static gpgpu_texture2D *tex=new gpgpu_texture2D(1024,1024);
	
	/* Make a GPU program to fill the texture with data. 
	  The program's return value is a color--
	*/
	static GLhandleARB prog=makeProgramObject(GPGPU_V,
		"varying vec4 location; /* onscreen location */\n"
		"uniform sampler2D table;\n"
		"void main(void) { /* look up texture */ \n"
		"	gl_FragColor = texture2D(table,location);\n"
		"}\n"
	);
	
	/* Plug the lookup table texture into our program object with this name */
	gpgpu_add(prog,table,"table",1);
	
	/* Execute the program into the texture, using a framebuffer */
	static gpgpu_framebuffer *fb=new gpgpu_framebuffer(tex);
	fb->run_time(prog);
	
	/* Show the resulting texture values */
	gpgpu_show_and_pause(tex,"GPGPU: Read Texture from GLSL");
	
	return 0;
}
